/*
 * vim:expandtab:shiftwidth=8:tabstop=8:
 *
 * Copyright CEA/DAM/DIF  (2008)
 * contributeur : Philippe DENIEL   philippe.deniel@cea.fr
 *                Thomas LEIBOVICI  thomas.leibovici@cea.fr
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 * ---------------------------------------
 */

/**
 * \file    mnt_Dump.c
 * \author  $Author: deniel $
 * \date    $Date: 2005/12/20 10:52:14 $
 * \version $Revision: 1.6 $
 * \brief   MOUNTPROC_Dump for Mount protocol v1 and v3.
 *
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef _SOLARIS
#include "solaris_port.h"
#endif

#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include <fcntl.h>
#include <sys/file.h>           /* for having FNDELAY */
#include "HashData.h"
#include "HashTable.h"
#include "log_macros.h"
#include "stuff_alloc.h"
#include "nfs23.h"
#include "nfs4.h"
#include "nfs_core.h"
#include "cache_inode.h"
#include "cache_content.h"
#include "nfs_exports.h"
#include "nfs_creds.h"
#include "nfs_tools.h"
#include "mount.h"
#include "nfs_proto_functions.h"

/**
 * mnt_Dump: The Mount proc Dump function, for all versions.
 *  
 *  @param parg        [IN]
 *  @param pexportlist [IN]
 *	@param pcontextp      [IN]
 *  @param pclient     [INOUT]
 *  @param ht          [INOUT] 
 *  @param preq        [IN] 
 *	@param pres        [OUT]
 *
 */

int mnt_Dump(nfs_arg_t * parg /* IN     */ ,
             exportlist_t * pexport /* IN     */ ,
             fsal_op_context_t * pcontext /* IN     */ ,
             cache_inode_client_t * pclient /* INOUT  */ ,
             hash_table_t * ht /* INOUT  */ ,
             struct svc_req *preq /* IN     */ ,
             nfs_res_t * pres /* OUT    */ )
{
  LogDebug(COMPONENT_NFSPROTO, "REQUEST PROCESSING: Calling mnt_Dump");

  /* Get the root of the mount list */
  pres->res_dump = nfs_Get_MountList();

  return NFS_REQ_OK;
}                               /* mnt_Null */

/**
 * mnt_Dump_Free: Frees the result structure allocated for mnt_Dump.
 * 
 * Frees the result structure allocated for mnt_Dump.
 * 
 * @param pres        [INOUT]   Pointer to the result structure.
 *
 */
void mnt_Dump_Free(nfs_res_t * pres)
{
  /* Nothing to do */
  return;
}                               /* mnt_Dump_Free */
